@@ -62,6 +62,8 @@ class Exams(object):
         basename = os.path.splitext(tex_path)[0]
         tmp_base_path = '%s_moodle_tmp'%(basename)
         tmp_path = tmp_base_path + '.tex'
+        xml_tmp_path = tmp_base_path + '.xml'
+        xml_output_file = basename + '.xml'
 
         file = open('students.csv', 'r')
         student_list = list(csv.reader(file, skipinitialspace=True))
@@ -77,7 +79,12 @@ class Exams(object):
             ncopies=n)
         dummy.update(dict(zip(extra_fields, extra_fields)))
         questions = self.question_list
-        ts = self.ts.data_surgery(dummy)
+        useamc = r'\usepackage[bloc,completemulti,pdfform,nowatermark]{automultiplechoice}'
+        _,lastpackage = self.ts.findall('\\usepackage{pkgname}')[-1]
+        ts = self.ts \
+            .insertAfter('\\usepackage{pkgname}[pkgname=%s]'%lastpackage['pkgname'], useamc) \
+            .data_surgery(dummy)
+
         for j,(qtype, qid, qtext) in enumerate(questions.values()):
             #copy and paste each exercise n-1 times, to create the n variants
             # each question has a different index, no need to do anything special
@@ -94,20 +101,23 @@ class Exams(object):
         with open(tmp_path, 'w') as tmp_file:
             tmp_file.write(tex_out)
         self._try_command( ['amc2moodle', tmp_path])
+        if os.path.exists(xml_output_file):
+            os.remove(xml_output_file)
+        os.rename(xml_tmp_path, xml_output_file)
+        for ext in ('.tex', '.log', '.aux', '.amc', '.out'):
+            if os.path.exists(tmp_base_path + ext):
+                os.remove(tmp_base_path + ext)
 
     def to_pdf(self, with_statement, with_solution, do_all):
-        tex_source, tex_path = self.src, self.path
-        basename = self.basename
-
         types = []
         if with_statement:
 #            statements_file = open('email_list_statements', 'w')
             statements_file = None
-            types.append(('question', 'false', statements_file))
+            types.append(('question', False, statements_file))
         if with_solution:
 #            solutions_file = open('email_list_solutions', 'w')
             solutions_file = None
-            types.append(('solution', 'true', solutions_file))
+            types.append(('solution', True, solutions_file))
         if do_all:
             for path,_,_ in types:
                 if os.path.exists(path):
@@ -119,27 +129,28 @@ class Exams(object):
     #        os.remove(data_file)
 
         file = open('students.csv', 'r')
-        student_list = list(csv.reader(file, skipinitialspace=True))
-        field_names = student_list[0]
+        field_names, *student_list = list(csv.reader(file, skipinitialspace=True))
         extra_fields = field_names[4:]
-
         if do_all:
-            student_list_to_process = student_list[1:]
             Parallel(n_jobs=8)(
-                delayed(self.one_pdf)(student, j, types, extra_fields, True)
-                for j,student in enumerate(student_list_to_process))
+                delayed(self.one_pdf)(
+                    student, j, types, extra_fields,
+                    do_all=True,
+                    ncopies=len(student_list))
+                for j,student in enumerate(student_list))
             # Non-parallel version
-            # for student in student_list_to_process:
-            #     one_pdf(student)
+            # for student in student_list:
+            #     one_pdf(student, ...)
         else:
             self.one_pdf(student_list[1], types=types, extra_fields=extra_fields)
 
     def one_pdf(self,
                 student,
                 index=0,
-                types=[('question', 'false', None)],
+                types=[('question', False, None)],
                 extra_fields=[],
-                do_all=False):
+                do_all=False,
+                ncopies=1):
         basename = self.basename
         name,surname,id,email,*extra_vals = student
         tmp_base_path = '%s_%s_tmp'%(basename,id)
@@ -147,13 +158,20 @@ class Exams(object):
         pdf_tmp_path = '%s_%s_tmp.pdf'%(basename,id)
         pdf_output_file = basename + '.pdf'
         student_seed = id
-        for (type, include_solution, output_file) in types:
+        for (type, is_solution, output_file) in types:
+            useamc = r'\usepackage[bloc,completemulti,pdfform,nowatermark%s]{automultiplechoice}'%(
+                ',correc' if is_solution else ''
+            )
+            _,lastpackage = self.ts.findall('\\usepackage{pkgname}')[-1]
+            self.src_for_pdf = TexSurgery(self.src)\
+                .insertAfter('\\usepackage{pkgname}[pkgname=%s]'%lastpackage['pkgname'], useamc)\
+                .src
             student_vars = dict(
                 name=name, surname=surname, id=id, seed=id, index=index,
-                include_solution=include_solution)
+                ncopies=ncopies)
             student_vars.update(dict(zip(extra_fields, extra_vals)))
             #TODO: add import to load our own sty instead of automultiplechoice
-            ts = TexSurgery(self.src)
+            ts = TexSurgery(self.src_for_pdf)
             ts.data_surgery(student_vars).code_surgery()
             #TODO: collect exam information
             tex_out = ts.src
@@ -170,7 +188,7 @@ class Exams(object):
                 if os.path.exists(pdf_output_file):
                     os.remove(pdf_output_file)
                 os.rename(pdf_tmp_path, pdf_output_file)
-            for ext in ('.tex', '.log', '.aux', '.amc'):
+            for ext in ('.tex', '.log', '.aux', '.amc', '.out'):
                 if os.path.exists(tmp_base_path + ext):
                     os.remove(tmp_base_path + ext)
     #    for _,_,file in TYPES:
